#include <ddraw.h>
#include <sys/timeb.h>
#include <stdio.h>
#include <math.h>

LPDIRECTDRAW7			m_lpDD;
LPDIRECTDRAWSURFACE7	m_lpDDSFront;
LPDIRECTDRAWSURFACE7	m_lpDDSBack;


HRESULT CreateSurface( DWORD size_x, DWORD size_y, LPDIRECTDRAWSURFACE7 *surface ) {
	HRESULT				dwResult;
	DDSURFACEDESC2		ddsd;

	ZeroMemory( &ddsd, sizeof( ddsd ) );
	ddsd.dwSize			= sizeof( ddsd );
	ddsd.dwFlags		= DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
	ddsd.dwWidth		= size_x;
	ddsd.dwHeight		= size_y;
	ddsd.ddsCaps.dwCaps	= DDSCAPS_OFFSCREENPLAIN;

	dwResult			= m_lpDD->CreateSurface( &ddsd, surface, NULL );
	return dwResult;
}

HRESULT InitDDraw( HWND hWnd, DWORD res_x, DWORD res_y, DWORD res_bit ) {
	m_lpDD			= NULL;
	m_lpDDSFront	= NULL;
	m_lpDDSBack		= NULL;

    HRESULT			dwResult;
	DDSURFACEDESC2	ddsd;
	DDSCAPS2		ddscaps;

    dwResult = DirectDrawCreateEx( NULL, ( void** )&m_lpDD, IID_IDirectDraw7, NULL );
    if ( dwResult != DD_OK ) {
        return dwResult;
    }

    dwResult = m_lpDD->SetCooperativeLevel( hWnd, DDSCL_EXCLUSIVE | 
													DDSCL_FULLSCREEN | 
													DDSCL_ALLOWREBOOT );
    if ( dwResult != DD_OK ) {
        return dwResult;
    }

    dwResult = m_lpDD->SetDisplayMode( res_x, res_y, res_bit, 0, 0 );
    if( dwResult != DD_OK ) {
        return dwResult;
    }

	ZeroMemory( &ddsd, sizeof( ddsd ) );
	ddsd.dwSize				= sizeof( ddsd );
	ddsd.dwFlags			=	DDSD_CAPS | 
								DDSD_BACKBUFFERCOUNT;
	ddsd.ddsCaps.dwCaps		=	DDSCAPS_PRIMARYSURFACE | 
								DDSCAPS_FLIP | 
								DDSCAPS_COMPLEX;
	ddsd.dwBackBufferCount	= 1	;

	dwResult = m_lpDD->CreateSurface( &ddsd, &m_lpDDSFront, NULL );
	if( dwResult != DD_OK ) {
		return dwResult;
	} 

    ZeroMemory( &ddscaps, sizeof( ddscaps ) );
    ddscaps.dwCaps = DDSCAPS_BACKBUFFER;

	dwResult = m_lpDDSFront->GetAttachedSurface( &ddscaps, &m_lpDDSBack );
    if( dwResult != DD_OK ) {
		return dwResult;
	}

	return dwResult;
}


void CleanDDraw() {
	if ( m_lpDD ) {
		if( m_lpDDSBack )	m_lpDDSBack->Release();
		if( m_lpDDSFront )	m_lpDDSFront->Release();		
		m_lpDDSFront	= NULL;
		m_lpDDSBack		= NULL;
		m_lpDD->RestoreDisplayMode();
		m_lpDD->Release();
		m_lpDD			= NULL;
	}
}

LRESULT WINAPI MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam ) {
    switch( msg ) {
        case WM_DESTROY:
            PostQuitMessage( 0 );
            return 0;

        case WM_PAINT:
            ValidateRect( hWnd, NULL );
            return 0;
    }

    return DefWindowProc( hWnd, msg, wParam, lParam );
}

void InitWin( WNDCLASSEX *wc, HWND *hWnd, char *class_name, char *win_name, int x1, int y1, int x2, int y2 ) {
    wc->cbSize			= sizeof(WNDCLASSEX);
	wc->style			= CS_CLASSDC;
	wc->lpfnWndProc		= MsgProc;
	wc->cbClsExtra		= 0L;
	wc->cbWndExtra		= 0L;
	wc->hInstance		= GetModuleHandle(NULL);
	wc->hIcon			= NULL;
	wc->hCursor			= NULL;
	wc->hbrBackground	= NULL;
	wc->lpszMenuName	= NULL;
	wc->lpszClassName	= class_name;
	wc->hIconSm			= NULL;

    RegisterClassEx(wc);

    ( *hWnd ) = CreateWindowEx( WS_EX_TOPMOST, class_name, win_name, 
                              WS_OVERLAPPEDWINDOW, x1, y1, x2, y2,
                              GetDesktopWindow(), NULL, wc->hInstance, NULL );
}

void CleanWin( WNDCLASSEX *wc, char *class_name ) {
	UnregisterClass( class_name, wc->hInstance );
}

/***************************************************************************/
#define RES_X 1024
#define RES_Y 768
#define RES_B 32
#define CLASS_N "Direct Draw Test"
#define WIN_N "Simulace"

#define MAX 2
#define ZOOM 0.000000000000001
//#define ZOOM 0.0000001
//#define ZOOM 0.001
#define ITER 2048

 
//#define POS_X -0.835 
#define POS_X -0.835000001 
#define POS_Y -0.2


struct	_timeb start, stop;
int		i = 0;
double	fps = 0;
char	sfps[20];
HDC		hdc;

void FPS() {
	if (m_lpDDSBack->GetDC(&hdc) == DD_OK) {
		sprintf(sfps, "FPS: %.1f", fps);
		SetBkColor( hdc, RGB( 0, 0, 0 ) );
		SetTextColor( hdc, RGB( 0, 0, 255 ) );
		TextOut( hdc, 0, 0, sfps, lstrlen(sfps) );
		m_lpDDSBack->ReleaseDC(hdc);
	}
	i ++;
	if (i > 50)
	{
		_ftime(&stop);
		fps = i/((stop.time - start.time) + 0.001*(stop.millitm - start.millitm));
		_ftime(&start);
		i = 0;
	}
}
void InitOther() {

	_ftime(&start);
	i = 0;
	fps = 0;
}

void CleanOther() {
}

int mandl(double re, double im) {
double  z;
double  re_d, im_d, _re;
int     opak;

  opak = 0;
  re_d = re;
  im_d = im;
  z = sqrt(re*re + im*im);
  while ((z < MAX)&&(opak < ITER))
   {
    _re = re*re - im*im + re_d;
    im = 2*re*im + im_d;
	re = _re;
    z = sqrt(re*re + im*im);
    opak++;
   }
//  if (opak<50) return(255);  else return(0);
  return(opak);
//  return((int)z);
}


void Render() {
}


INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, INT ) {
	WNDCLASSEX	wc;
	HWND		hWnd;
	BOOL		gm = TRUE;
	MSG			msg;

	InitWin( &wc, &hWnd, CLASS_N, WIN_N, 100, 100, 200, 200 );

    if( SUCCEEDED( InitDDraw( hWnd, RES_X, RES_Y, RES_B ) ) ) {
		InitOther();
        ShowWindow( hWnd, SW_SHOWDEFAULT );
        UpdateWindow( hWnd );


		HBRUSH	hb	= CreateSolidBrush(RGB(0, 0, 0));
		RECT	obd = {0, 0, RES_X, RES_Y};
		int x = 0, y = 0;
		bool fr = true;
		while(gm) {
			if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) ) {
				if( gm = GetMessage( &msg, NULL, 0, 0 ) ) {
					TranslateMessage( &msg );
					DispatchMessage( &msg );
				}
			}
			else if (fr) {
				if (m_lpDDSFront->GetDC(&hdc) == DD_OK) {
					for (y = 0; y < RES_Y; y ++) {
						SetPixel(hdc, x, y, RGB(0, (double)256/ITER*mandl((x-RES_X*0.5)*ZOOM + POS_X, (y-RES_Y*0.5)*ZOOM + POS_Y), 0));
					}
					x ++;
					if (x >= RES_X) fr = false;
					m_lpDDSFront->ReleaseDC(hdc);
				}
			}
		}
		
    }

    CleanOther();
    CleanDDraw();
    CleanWin(&wc, CLASS_N);

    return 0;
}
